package com.shanghaichengdi.ghjgitem.service.impl;

import com.github.pagehelper.PageHelper;
import com.shanghaichengdi.ghjgitem.enums.SupervisionAreaCode;
import com.shanghaichengdi.ghjgitem.mapper.CommonMapper;
import com.shanghaichengdi.ghjgitem.service.CommonService;
import com.shanghaichengdi.ghjgitem.vo.BoatAISVo;
import com.shanghaichengdi.ghjgitem.vo.BoatReportInfoVo;
import com.shanghaichengdi.ghjgitem.vo.CCbCargoDeclareRecVo;
import com.shanghaichengdi.ghjgitem.vo.CCbOnShipCrewRecVo;
import com.shanghaichengdi.ghjgitem.vo.PageVo;
import com.shanghaichengdi.ghjgitem.vo.ShipDataVo;
import com.shanghaichengdi.ghjgitem.vo.VwCertInfoSelfAllVo;
import com.shanghaichengdi.ghjgitem.vo.request.BoatAisRealTimeReq;
import com.shanghaichengdi.ghjgitem.vo.request.ReportInfoRequest;
import com.shanghaichengdi.ghjgitem.vo.request.ReportSubInfoReq;
import com.shanghaichengdi.ghjgitem.vo.request.ShipDataRequest;
import com.shanghaichengdi.ghjgitem.vo.request.SupervisionOrDocNumberReq;
import com.shanghaigeography.Util.DateUtils;
import java.text.ParseException;
import java.time.LocalDate;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import javax.annotation.Resource;
import org.springframework.stereotype.Service;

@Service
public class CommonServiceImpl implements CommonService {

  @Resource
  CommonMapper mapper;

  @Override
  public List<BoatAISVo> selectAisRealTimeXYList(BoatAisRealTimeReq param) {
    PageHelper.startPage(param.getPageIndex(), param.getPageSize());
    return mapper.selectAisRealTimeXYList(param);
  }

  @Override
  public String selectRecordOrDocNumber(SupervisionOrDocNumberReq param) {
    // 港航行业监管告知书编码规则为：区代码+年份+五位流水号。
    String areaCode = SupervisionAreaCode.getByCode(param.getAreaCode()).getCode();
    Integer year = LocalDate.now().getYear();
    String codeInOracle = null;
    StringBuilder supervisionOrDocNumber = null;
    String rules = areaCode + year;
    if (!Objects.isNull(param.getStationCode())) {
      rules = rules + param.getStationCode();
    }
    if ("inform".equals(param.getType())) {
      codeInOracle = mapper.getLastInformNo(rules);
    } else if ("record".equals(param.getType())) {
      codeInOracle = mapper.getLastRecordNo(rules);
    } else {
      throw new RuntimeException("类型输入有误");
    }
    if (Objects.isNull(codeInOracle)) {
      codeInOracle = rules;
      // 根据实际规则，除最后一位之外都补0
      for (int i = 0; i < 11 - rules.length() - 1; i++) {
        codeInOracle = codeInOracle + "0";
      }
      codeInOracle = codeInOracle + "1";
      return codeInOracle;
    }
    Integer next = Integer.parseInt(codeInOracle.substring(rules.length())) + 1;
    supervisionOrDocNumber = new StringBuilder(rules);
    // 根据实际规则，除最后一位之外都补0
    for (int i = 0; i < 11 - rules.length() - String.valueOf(next).length(); i++) {
      supervisionOrDocNumber.append("0");
    }
    supervisionOrDocNumber.append(next);
    return supervisionOrDocNumber.toString();
  }

  @Override
  public List<CCbCargoDeclareRecVo> selectCargoDeclareInfo(ReportSubInfoReq param) {
    PageHelper.startPage(param.getPageIndex(), param.getPageSize());
    return mapper.selectCargoDeclareInfo(param);
  }

  @Override
  public List<CCbOnShipCrewRecVo> selectShipCrewInfo(ReportSubInfoReq param) {
    PageHelper.startPage(param.getPageIndex(), param.getPageSize());
    return mapper.selectShipCrewInfo(param);
  }

  @Override
  public List<VwCertInfoSelfAllVo> selectCertInfoList(PageVo param) {
    PageHelper.startPage(param.getPageIndex(), param.getPageSize());
    return mapper.selectInfoList();
  }

  @Override
  public List<ShipDataVo> selectBoatInfoList(ShipDataRequest param) {
    PageHelper.startPage(param.getPageIndex(), param.getPageSize());
    return mapper.selectBoatInfoList(param);
  }

  @Override
  public List<BoatReportInfoVo> selectBoatReportInfoList(ReportInfoRequest param)
      throws ParseException {
    String startTime = param.getQueryDate() + " 00:00:00";
    String endTime = param.getQueryDate() + " 23:59:59";
    String partition = "SHIP_REPORT_" + DateUtils.dateToString(
        DateUtils.stringToDate(param.getQueryDate(), DateUtils.DATE_FORMAT_10),
        DateUtils.DATE_FORMAT_11);
    PageHelper.startPage(param.getPageIndex(), param.getPageSize());
    return mapper.selectBoatReportInfoList(param.getReportId(), "",
        param.getDistrict(), param.getShipId(), startTime, endTime, partition, param.getReportNo());
  }
}
